@@ -22,8 +22,8 @@ def optimized_portfolio_on_sharperatio(r, cov, rf, minweight=0.):
     )
 
 
-def optimized_portfolio_expectation_maximization(r, cov, rf, V0, c):
-    func = partial(mpt_costfunction, r=r, cov=cov, rf=rf, V0=V0, c=c)
+def optimized_portfolio_mpt_costfunction(r, cov, rf, lamb, V0=10.):
+    func = partial(mpt_costfunction, r=r, cov=cov, rf=rf, lamb=lamb, V0=V0)
     nbstocks = len(r)
     constraints = [
         LinearConstraint(np.eye(nbstocks+1), 0, V0)
